searchData={"items":[{"type":"module","title":"Spannex","doc":"The entry module for interacting with `Spannex.Protocol`, the underlying `DBConnection` implementation.\n\nTo create a Spannex database connection to Cloud Spanner, simply call `Spannex.start_link/1` with the database configuration or pass in `{Spannex, opts}` to your supervisor.","ref":"Spannex.html"},{"type":"module","title":"Queries vs Mutations - Spannex","doc":"Spanner makes a clear distinction between \"queries\", which are read-only statements, and \"mutations\", which are statements that modify the state of the database. However, Spannex still provides a single function, `execute/4`, and will perform any necessary internal conversions depending on the type of the statement being executed.","ref":"Spannex.html#module-queries-vs-mutations"},{"type":"module","title":"Named Variables - Spannex","doc":"Currently, only named variables are accepted in queries and mutations. You must also pass in a map of named variables when executing a statement that contains named variables. For example, if you have a query like `SELECT * FROM users WHERE id = @id`, you must pass in a map containing `%{id: 1}` when executing the statement.","ref":"Spannex.html#module-named-variables"},{"type":"function","title":"Spannex.child_spec/1","doc":"Returns a child specification for the Spannex database connection using the given options.\n\nYou *must* supply at least one option, `:database`, to specify the Cloud Spanner database to connect to.\n\nIt is recommended to avoid supplying the `:name` option and to instead let the connection pool handle fetching and returning individual connections.\n\nSee `Spannex.Protocol.connect/1` for a full list of accepted options.","ref":"Spannex.html#child_spec/1"},{"type":"function","title":"Spannex.execute/4","doc":"Converts a string SQL statement into a `Spannex.Query` struct before calling `DBConnection.prepare_execute/4`.","ref":"Spannex.html#execute/4"},{"type":"function","title":"Spannex.start_link/1","doc":"Starts a Spannex database connection using the given options.\n\nSynonymous with `DBConnection.start_link(Spannex.Protocol, opts)`.\n\nYou *must* supply at least one option, `:database`, to specify the Cloud Spanner database to connect to.\n\nSee `Spannex.Protocol.connect/1` for a full list of accepted options.","ref":"Spannex.html#start_link/1"},{"type":"function","title":"Spannex.transaction/3","doc":"Runs the given function inside a Spanner transaction.","ref":"Spannex.html#transaction/3"},{"type":"module","title":"Spannex.Protocol","doc":"A `DBConnection` implementation for Cloud Spanner.","ref":"Spannex.Protocol.html"},{"type":"module","title":"Connections - Spannex.Protocol","doc":"Each connection maintains its own gRPC channel to the Spanner API and its own unique session. The session is used as a unique identifier for all other requests made against the database.","ref":"Spannex.Protocol.html#module-connections"},{"type":"module","title":"Transactions - Spannex.Protocol","doc":"When starting a transaction, there are four accepted types of transactions:\n\n1. `:batch_write` - Uses \"batch write\" and \"mutation groups\" to execute multiple operations in a single transaction, optimized for \"blind bulk writes\", low-latency, and atomicity within each group of mutations.\n2. `:read_write` - A locking read-write transaction, this is what you would consider a \"traditional transaction\". It's best used when you are changing one row or a handful of related rows or need to reference data created in the same transaction.\n3. `:read_only` - A special transaction type that allows for point-in-time consistency in the data being read. This helps eliminate possible drift in values when operations might be happening in parallel to the read-only transaction by locking the state of the data being read to a specific point in time.\n4. `:partitioned_dml` - A special transaction where regular DML statements are partitioned and executed in parallel. This is useful for large-scale bulk operations that still need guarantees of atomicity and consistency.\n\nThe `:batch_write` transaction differs from `:read_write` and `:partitioned_dml` transactions in some important ways:\n\n1. It does not support reading data. It is strictly for writing data.\n2. You cannot reference changes made by other mutations within the same transaction.\n3. You must specify all literals explicitly, you cannot use named query parameters.\n4. All queries are locally queued and then sent all at once when the transaction is committed. This means individual statements are not sent to the database, rather they are batched into a single gRPC request when the transaction is comitted.\n5. Deletions *must* be done by primary key. If you need a custom where-clause to delete rows, you must do so in a `:read_write` or `:partitioned_dml` transaction using regular DML statements.\n\n#","ref":"Spannex.Protocol.html#module-transactions"},{"type":"module","title":"Timeouts - Spannex.Protocol","doc":"Transactions have a 10-second idle timeout that cannot be modified. If a transaction remains idle for more than 10 seconds, it is aborted. Aborted transactions cause the session to enter a limbo state where you can't really do anything useful anymore, so the default behavior is to disconnect and force the creation of a new connection.\n\nWith the exception of a `:batch_write` transactions, all individual statements resolved via `handle_execute` will reset the 10-second idle timer. But if you start a `:batch_write` transaction, you have 10 seconds to queue up all necessary statements and then commit the transaction. While this should be sufficient in most scenarios, it is worth keeping this in mind if you are using `:batch_write` transactions for very large operations. In such cases where the 10-second idle timeout might be a problem, consider pre-calculating as much data for the DML statements as possible and then starting a `:batch_write` transaction after.","ref":"Spannex.Protocol.html#module-timeouts"},{"type":"module","title":"Queries - Spannex.Protocol","doc":"Spanner requires that all write operations, i.e. queries that are not `SELECT` statements, use an explicit transaction. If you attempt to execute a write statement outside of a transaction, you will receive an error.\n\nFor read queries, i.e. `SELECT` statements, a temporary read-only transaction with a strong consistency guarantee can be created for you.\n\nSpanner also stores version history for all columns and all rows and all tables, meaning you can get truly strong consistency guarantees when reading data. You can ensure that, given a specific (recent) point in time, all data read from the database is no more recent than that point in time.","ref":"Spannex.Protocol.html#module-queries"},{"type":"function","title":"Spannex.Protocol.checkout/1","doc":"No-op","ref":"Spannex.Protocol.html#checkout/1"},{"type":"function","title":"Spannex.Protocol.connect/1","doc":"Start a new gRPC connection to the Spanner API and also generate a new session.\n\nThere is only one required option for the `connect/1` function: `:database`. This tells Spannex which database the session will be for.\n\nIf you do not supply an authorization header as part of `:grpc_opts`, you must also supply the `:goth` option, which tells Spannex how to fetch an access token.\n\nYou may optionally supply the following other options:\n\n- `:goth` - The term used to access the `Goth` library to fetch an access token.\n- `:labels` - A map of labels to apply to the session. These labels can be used to filter and monitor sessions using the Cloud console or admin API.\n- `:host` - The gRPC endpoint to connect to. If not supplied, the global Spanner API endpoint is used.\n- `:grpc_opts` - The options to pass to `GRPC.Stub.connect/2`.\n- `:cred` - A gRPC credential wrapper. If not supplied, the `Goth` library is used to fetch a Google Cloud access token.\n- `:headers` - A list of headers to use for the connection. If `content-type` and `authorization` are supplied, they will be overwritten.","ref":"Spannex.Protocol.html#connect/1"},{"type":"function","title":"Spannex.Protocol.convert_value/3","doc":"Converts a single column value from its gRPC-encoded form to its Elixir-native representation. The `type` parameter is the native type of the column and the `enc_type` parameter is the type of the encoded value.","ref":"Spannex.Protocol.html#convert_value/3"},{"type":"function","title":"Spannex.Protocol.decode_results/1","doc":"Converts the row data from a `Google.Spanner.V1.ResultSet` into a list of maps, where each map is one row of the result set. The keys of the map are the column names and the values are the column values. The column values are converted from their encoded form to their Elixir-native representation.","ref":"Spannex.Protocol.html#decode_results/1"},{"type":"function","title":"Spannex.Protocol.decode_row/3","doc":"","ref":"Spannex.Protocol.html#decode_row/3"},{"type":"function","title":"Spannex.Protocol.handle_begin/2","doc":"Begins a new transaction. You *must* use a transaction for any mutations, i.e. anything that isn't a `SELECT` query.\n\nMay specify the option `:type` as any one of: `:batch_write`, `:read_write`, `:read_only`, `:partitioned_dml`. If `:type` is not set, it defaults to `:read_write`.\n\nIf `:batch_write` is specified, all subsequent `handle_execute/4` calls will simply queue up the statements locally in a list that will be converted to a set of mutations when `handle_commit/2` is called. If `handle_rollback/2` is called instead, the queue is simply cleared, nothing happens remotely.","ref":"Spannex.Protocol.html#handle_begin/2"},{"type":"function","title":"Spannex.Protocol.handle_close/3","doc":"No-op","ref":"Spannex.Protocol.html#handle_close/3"},{"type":"function","title":"Spannex.Protocol.handle_commit/2","doc":"Commits a running transaction.\n\nIf the transaction is a `:batch_write` transaction, all queued statements are converted to \"mutations\" and sent to the Spanner API in a single request.","ref":"Spannex.Protocol.html#handle_commit/2"},{"type":"function","title":"Spannex.Protocol.handle_deallocate/4","doc":"Would normally deallocate a cursor generated by `handle_declare/4`, but we don't support cursors in this implementation.","ref":"Spannex.Protocol.html#handle_deallocate/4"},{"type":"function","title":"Spannex.Protocol.handle_declare/4","doc":"Would normally generate a cursor for a query prepared by `handle_prepare/3`, but we don't support cursors in this implementation.","ref":"Spannex.Protocol.html#handle_declare/4"},{"type":"function","title":"Spannex.Protocol.handle_execute/4","doc":"Executes a SQL statement against the Spanner API.\n\nAll statements that are mutations (i.e. not `SELECT` statements) must be executed within a transaction or they will produce an error.\n\nIf a `SELECT` statement is executed outside of a transaction, the database will generate a temporary `:read_only` transaction to run it in.\n\nIf the transaction type is `:batch_write`, the query will be queued up locally and not sent to the Spanner API until `handle_commit/2` is called. This means the result portion of the response tuple will be `nil` since there's no result to return yet.","ref":"Spannex.Protocol.html#handle_execute/4"},{"type":"function","title":"Spannex.Protocol.handle_fetch/4","doc":"Would normally fetch the next result from a cursor generated by `handle_declare/4`, but we don't support cursors in this implementation.","ref":"Spannex.Protocol.html#handle_fetch/4"},{"type":"function","title":"Spannex.Protocol.handle_prepare/3","doc":"No-op","ref":"Spannex.Protocol.html#handle_prepare/3"},{"type":"function","title":"Spannex.Protocol.handle_rollback/2","doc":"Rolls back a running transaction.\n\nIf the transaction is a `:batch_write` transaction, all queued statements are simply cleared and the state is returned to idle.","ref":"Spannex.Protocol.html#handle_rollback/2"},{"type":"function","title":"Spannex.Protocol.make_mutation/4","doc":"Creates a mutation struct for the given operation. For anything other than a `:delete` operation, the `columns` and `rows` parameters are used to create a `Google.Spanner.V1.Mutation.Write` struct.\n\nFor a `:delete` operation, the Spanner API expects that the primary key will be used to execute the delete mutation. If you need to delete using a where-clause and not the primary key, you cannot use a mutation. The `rows` parameter is expected to be a list of values that correspond to the primary key columns in the same order as they are defined in the table schema.","ref":"Spannex.Protocol.html#make_mutation/4"},{"type":"function","title":"Spannex.Protocol.ping/1","doc":"Performs a `GetSession` request to ensure the session is still valid and operational.","ref":"Spannex.Protocol.html#ping/1"},{"type":"function","title":"Spannex.Protocol.query_to_mutation/1","doc":"Parses a SQL query with optional named parameters and converts the operation to a mutation using `make_mutation/4`.","ref":"Spannex.Protocol.html#query_to_mutation/1"},{"type":"type","title":"Spannex.Protocol.t/0","doc":"","ref":"Spannex.Protocol.html#t:t/0"},{"type":"type","title":"Spannex.Protocol.transaction_type/0","doc":"","ref":"Spannex.Protocol.html#t:transaction_type/0"},{"type":"module","title":"Spannex.Query","doc":"Handles parsing and preparing queries for use by the Spanner database connection protocol.","ref":"Spannex.Query.html"},{"type":"type","title":"Spannex.Query.t/0","doc":"","ref":"Spannex.Query.html#t:t/0"},{"type":"module","title":"Spannex.Service","doc":"A wrapper around the gRPC/Protobuf service stubs.\n\nDoes not necessarily support the entire set of stubbed RPC functions. Will add per-call metadata to the requests as needed.\n\n**Note**: In the future this module should use `:telemetry` to emit spans for each gRPC request. This would allow applications to understand how long various requests take and also track things like the number of errors and whether they correlate to particular sessions or request types.","ref":"Spannex.Service.html"},{"type":"function","title":"Spannex.Service.execute/2","doc":"Executes a Spanner gRPC request. Per-request metadata may also be added to the gRPC request.","ref":"Spannex.Service.html#execute/2"},{"type":"extras","title":"Spannex","doc":"# Spannex\n\nElixir client for the Google Cloud Spanner database using gRPC.","ref":"readme.html"},{"type":"extras","title":"Why? - Spannex","doc":"There's really only one option to use Google Cloud Spanner with Elixir right now, and that's the official Google Cloud API version that hasn't been updated in 2 years and uses REST rather than gRPC. What's more, it makes it very difficult to build an Ecto Adapter on top of.\n\nThis library instead chooses to implement a Spanner API via [DBConnection](https://hexdocs.pm/db_connection/DBConnection.html) and uses gRPC as its connection mechanism.\n\nAt the moment, each individual connection generates its own \"channel\" rather than sharing one. This is technically not necessary, as Spanner uses *sessions* to determine \"who\" is performing which operations, not the gRPC socket channel. But the complexity of generating fewer channels and sharing them is a bit more than needed at this time, and it can be an improvement down the road if needed.","ref":"readme.html#why"},{"type":"extras","title":"Getting Started - Spannex","doc":"Add the library to your project:\n\n```elixir\ndefp dependencies() do\n  {:spannex, \"~> 0.1\"}\nend\n```\n\nThen you just start it like you would any other `DBConnection`: using `start_link/2` manually or by passing a child spec to a supervisor. The latter is probably more common, and it is recommended to use `DBConnection.child_spec` to create an appropriate child spec.\n\n```elixir\nDBConnection.child_spec(Spannex.Protocol, [\n  name: MyApp.Spanner,\n  pool_size: 5,\n  database: \"projects/p/instances/i/databases/d\",\n])\n```\n\nThe only *required* option is `:database`. Notice how the example above uses the fully-qualified database name. If you don't set this, or don't use the appropriate format like is shown above, you'll get an error. Not setting it will result in a runtime error where an invalid database name will result in an error tuple.\n\nBy default, `Spannex` will use the global Spanner API endpoint for new connections: `spanner.googleapis.com`. If you need to ensure connections are regional for whatever reason, you can manually override that in your child spec by setting `:host` in the options.\n\nAdditionally, `Spannex` uses [goth](https://github.com/peburrows/goth) to fetch the appropriate Google Cloud Credentials to authenticate when opening a gRPC channel to the API. Ensure that whatever user or service account credentials you use have the appropriate permissions to interact with the Spanner database.\n\n*Note*: the Spanner API must be enabled in the project your application is running in, even if the Spanner instance itself is in another project. Failing to have the Spanner API enabled in the application project will result in a runtime error.\n\nAnd because we use [grpc](https://github.com/elixir-grpc/grpc) to communicate with the database, you can also fine-tune you gRPC configuration by setting `:grpc_opts` in the child spec options.","ref":"readme.html#getting-started"},{"type":"extras","title":"Querying - Spannex","doc":"Right now only \"simple\" querying is supported. The query wrapper, `Spannex.Query`, only has two fields: `:statement` and `:params`. When a statement is executed immediately, i.e. in a `:read_write` transaction during a `handle_execute/4` callback, the supplied parameters completely override the encoded `Spannex.Query` parameters. In fact, the encoded query parameters are only ever used during `:batch_write` transaction commits, since all of the SQL statements for batch write mutations are queued locally and \"executed\" only during the commit phase.","ref":"readme.html#querying"},{"type":"extras","title":"Protobuf - Spannex","doc":"The repo contains a \"pinned\" version of the necessary Protobuf files required to successfully run `Spannex`.\n\nWhile it is possible to build the protobuf output dynamically, they don't change often. And even if they did change, we would want to pin the version that works for us until we could upgrade safely. Therefore it makes sense to do the generation once and then save the output to the repo for easier/faster application building.\n\nThere might actually be *more* protobuf files generated than are actually the minimum set necessary to run this project. The list might be trimmed down over time to avoid unnecessary code in the repo, but right now I basically just generated `*.pb` to ensure I had everything I would to get this thing working properly. And the admin protobuf files are useful because if someone uses `Spannex` for their project, they might also have a need for the admin API, and so they could set up their own gRPC connection and already have all of the admin protobuf files at the ready.","ref":"readme.html#protobuf"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}